<?php if ( !defined('BASEPATH') ) die('No direct script access allowed');

/**-------------------------------------------------------------------------
 *
 *	MAuth Installation Controller
 *	
 *	If this is your first install, meaning no database tables are installed,
 *	then you will be routed to /install/
 *
 * @file:  		./application/controllers/install.php
 * @version:	0.9.2
 * @modified:	October, 2010
 * @author:		Pat Andrew	(http://patandrew.com/)
 * @link:		http://mauth.missionsix.net/
 *
 *-------------------------------------------------------------------------*/

 
DEFINE('VIEWS', realpath(dirname(__FILE__)) . '/INSTALL/views/');
 
class Install extends Controller
{
 	
 	// page data store
 	private $data;
 	
 	private $errors;
 	
 	// config
 	private $conf;
 	
 	
 	//	Upgrade SQL path
 	//	This array stores the upgrade .sql file path for table schema and
    //  database upgrade instructions.
 	private $upgradePath = array(
 			'new' 	=> 'INSTALL/sql/mauth_tables_new.sql',
 			'0.5.0' => 'INSTALL/sql/mauth_tables_0.5.0.sql',
  			'0.5.1' => 'INSTALL/sql/mauth_tables_0.5.1.sql',
  			'0.5.2' => 'INSTALL/sql/mauth_tables_0.5.2.sql',
  			'0.6.0' => 'INSTALL/sql/mauth_tables_0.6.0.sql',
  			'0.6.1' => 'INSTALL/sql/mauth_tables_0.6.1.sql',
 			'0.8.1' => 'INSTALL/sql/mauth_tables_0.8.1.sql',
            '0.9.0' => 'INSTALL/sql/mauth_tables_0.9.0.sql',
            '0.9.1' => 'INSTALL/sql/mauth_tables_0.9.1.sql',
            '0.9.2' => 'INSTALL/sql/mauth_tables_0.9.2.sql'
		);

 	/**
 	 *	Constructor
 	 *	sets up the current version controller and handles 
	 *	installation checks
	 * 
	 * @post: installation initialized.  Version checked
 	 */
 	function __construct() {

        parent::__construct();
 		
        $this->load->add_package_path(APPPATH.'third_party/mauth/');

 		$this->load->config('mauth');
        $this->load->helper('mauth');
        $this->load->model('users_model');

        $this->load->remove_package_path();

		$this->load->helper(array('url','cookie'));
		
		$this->conf = $this->config->item('mauth');
		
		if( ! $this->_check_config() )
		{
			$this->data['pagetitle'] = "Invalid Mauth Config";
			
			$this->data['next'] = array('', 'Reload Installer');
			$this->data['body'] .= $this->_load(VIEWS .'nextlink.php');
			$this->display();
			exit;	
		}

		$this->data['pagetitle'] = 'Mauth '. MAUTH_VERSION .' Installer';	

		$this->_currentVersion();
 	}	

	
	/**
	 *	Installer Index
	 *	determines if we are upgrading or new install
	 *
	 */
	function index()
	{	
			if(CURRENT_VERSION)
			{
				if(CURRENT_VERSION == MAUTH_VERSION)
				{
					//done!
					redirect('install/complete');	
				}
				elseif(CURRENT_VERSION < MAUTH_VERSION)
				{			
					$this->data['next'] = array('step/'. NEXT_VERSION, 'Start Upgrade');
					$this->data['body'] = $this->_load(VIEWS .'start_upgrade.php');
				}
			} else {
				// new install
				$this->data['next'] = array('step/new','Begin Installation');
				$this->data['body'] = $this->_load(VIEWS .'start_install.php');	
			
			}
			
			$this->data['body'] .= $this->_load(VIEWS .'nextlink.php');
			
		$this->display();
	}
	
	
	/**
	 *	Main Step Function
	 *	routes and modifies the steps.
	 *
	 */
	function step()
	{
		$step = $this->uri->segment(3);
		
		if($step == 'new')
		{
			$this->data['next'] = array('createadmin','Continue, Create Admin Account');
		}
		else
		{
			// find the next upgrade path.
			$this->data['next'] = array('install','Continue Upgrade');
			
		}
		
		print $this->_load(VIEWS . 'header.php');
 		
 		print '<h2>Reading file <strong>'. $this->upgradePath[$step] .'</h2>';
 		
		$this->_processSqlFile($this->upgradePath[$step]);
 		
 		
 		print $this->_load(VIEWS . 'nextlink.php');
 		print $this->_load(VIEWS . 'footer.php');
	}
	
	
	/**
	 *	Create Administrative Account
	 *	this method provides a form to create the administrative account.
	 *
	 *	Upon successful data, we insert into database, and redirect to install/complete
	 */
	 
	function createadmin()
	{
           // load requirements
 
		$this->load->add_package_path(APPPATH .'/third_party/mauth/');
        $this->load->library( array('form_validation', 'mauth') );
        $this->load->model('users_model');   
        $this->load->remove_package_path();

		$this->form_validation->set_rules('username', 'Username', 'trim|required'.
				(isset($this->conf['min_username_length']) ? '|min_length['.$this->conf['min_username_length'] .']' : '') 
				.(isset($this->conf['max_username_length']) ? '|max_length['.$this->conf['max_username_length'] .']' : '')
				.'|xss_clean');
		$this->form_validation->set_rules('password', 'Password', 'trim|required|matches[passconf]|callback_generate_password');
		$this->form_validation->set_rules('passconf', 'Password Confirmation', 'trim|required');
		$this->form_validation->set_rules('email', 'Email', 'trim|required|valid_email');
		
		$this->form_validation->set_error_delimiters('<p class="error">', '</p>');
		
		if($this->form_validation->run() === false) {
			
			print $this->_load(VIEWS . 'header.php');
			print $this->_load(VIEWS .'addadmin.php');
			print $this->_load(VIEWS . 'footer.php');
		} else {
		
            
			
			$admin = array(
							'username' => $this->input->post('username'),
							'password' => $this->input->post('password'),
							'email'    => $this->input->post('email'),
							'roleid'   => '2'
							);
			
			$this->users_model->add_user($admin);
			
			//redirect
			redirect('install/complete');
		}
		
	}
	 
	/** 
	 *	Install Complete
	 */ 	
 	function complete()
	{
 		$this->data['body'] = $this->_load(VIEWS .'complete.php');
 		$this->display();
 	}	
 	
 	
 	/**
 	 *	Display installation page
 	 */
 	function display()
	{
 		print $this->_load(VIEWS . 'header.php');
 		
 		print $this->data['body'];
 		
 		print $this->_load(VIEWS . 'footer.php');
 	}
 	
 	
 	
######################################################################
##
##		Helper Functions
##	
###################################################################### 	
 	
 	/**
 	 *	Check Config
 	 *  checks to make sure all config values are properly set before continuing the install
 	 *
 	 */
 	function _check_config() 
 	{
 		
 		if( ! $this->config->item('enable_hooks') )
 		{
			$this->data['body'] = $this->_load(VIEWS .'hooks.php');	
 			
 			return false;
 		}
 		if ( ! defined("MAUTH_VERSION") )
 		{
 			define('MAUTH_VERSION', 'unknown');
 			
 			$this->data['body'] = $this->_load(VIEWS . 'bad_subclassing.php');
 			
 			return false;
 		}
 		
		/* if ( ! function_exists("_continue_mauth") )
		{
 			$this->data['body'] = $this->_load(VIEWS . 'bad_config.php');
 			
 			return false;
 		}
 		*/
 		
 		return true;
 	}
 	
 	/** 
 	 *	Current Version 
 	 *	
 	 */
 	function _currentVersion()
	{
 	 	if($this->db->table_exists($this->conf['table_prefix'] .'store'))
		{
 	 		$res = $this->db->query('SELECT data FROM `'. $this->conf['table_prefix'] .'store` WHERE `key` LIKE \'version\'');
  			
  			$ver = $res->row_array();
  			
  			if(!empty($ver))
			{
  				define('CURRENT_VERSION', $ver['data']);
  				
  				//find the next version.
				$versions = $this->upgradePath;
				$nextversion = false;
				while ($version = current($versions))
				{
					if(key($versions) == CURRENT_VERSION)
					{
						next($versions);
						$next_version = key($versions);
						prev($versions);
					}
					next($versions);
				}
				
				define('NEXT_VERSION', $next_version);
  				  				
 	 		} else {
 	 			define('CURRENT_VERSION', false);
 	 		}
 	 	} else {
 	 		define('CURRENT_VERSION', false);
 	 	}
 	 }

 	/** 
 	 *	@private  processSqlFile
 	 *	
 	 *	prints progress of building tables, halts if SQL file is bad.
 	 */
 	function _processSqlFile($filename = '')
	{
 		if(empty($filename)) return FALSE;
 		
        $this->data['DEFAULT_USERID'] = $this->conf['user_roleid'];
 		$this->data['TABLE_PREFIX'] = $this->conf['table_prefix'];
 		
        $sqlFile = $this->_load($filename);
        
     	$sqlArray = explode(';',$sqlFile);
 		//Process the ;sql file by statements
     	foreach ($sqlArray as $stmt)
		{
            if( preg_match_all('/### (.*) ###/', $stmt, $matches ) )
            {
                print '<pre>'. $matches[1][0] ."...\t";
   			    if (strlen($stmt)>3)
			    {
        		    $result = $this->db->query($stmt);
        	
        		    if($result)
        			    print "Done!";
        		    else
                        print "FAIL!";
       		    }
       		    print '</pre>';
            }	
      	}

 		return; 
 	}
 	
 	
 	/**
 	 *	Generates a hashed password	
 	 *	used in createadmin
 	 */
 	function generate_password($password) {
		if( empty($password) ) return FALSE;

		$hash = $this->mauth->make_password($password);

		return $hash;
	} 
	
 	/**
 	 *	Load File
 	 *	returns a string with the file contents.
 	 *
 	 *	returns false if failed.
 	 */
 	function _load($filename = '')
	{
 		if(empty($filename)) return FALSE;

 		$data = $this->data;
 		
 		// load the php file contents
		
		foreach($this->data as $key => $value)
		{
			$$key = $value;
		}
    	
    	ob_start();
  			include $filename;
  			$result = ob_get_clean();

        // ob_clean();

 		foreach($this->data as $key => $value)
		{
			unset($$key);
		}

		return $result;
 	} 
 	
 	/**
 	 *	Display Errors
 	 *	outputs $this->errors in a dev friendly format
 	 */
 	private function _displayerrors()
	{
 		print '<hr /><div style="background:#efefef; padding:20px; margin:20px;"><h3>Installation Errors:</h3>';
 		if(!empty($this->errors)) {
 			print '<pre>';
	 		foreach($this->errors as $error) {
	 			print $error .'';
	 		}
	 		print '</pre>';
	 	} else {
	 		print 'There were no errors during installation.';
 		}
 		print '</div>';
 	}
 }

/*	end of file:	install.php
	file location:	./application/controllers/install.php	*/
