<?php if ( !defined('BASEPATH') ) die('No direct script access allowed');
/**-------------------------------------------------------------------------
 *
 *	Users Controller
 *	Process users login, registration, profile, logout
 * 
 * Mauth
 * missionsix authentication checking
 *
 * @file:  		./application/controllers/install.php
 * @version:	0.9.2
 * @modified:	October, 2010
 * @author:		Pat Andrew	(http://patandrew.com/)
 * @link:		http://mauth.missionsix.net/
 *
 *-------------------------------------------------------------------------*/

class Users extends Mauth_Controller
{

	/**
	 * Constructor
	 * Builds template info and sets up controller specific navigation
	 *
	 * @access:	public
	 * @post:	mauth loaded
	 */
	public function __construct()
	{
		parent::__construct();
		
        $this->load->helper('html');

		// build users navigation
		
		$nav = array('Members' 			=> 'users/',
					 'Who\'s Online' 	=> 'users/online'
					 );
		
		if( $this->permissions->CAN_Edit_Roles ) {
			$nav['Roles'] = 'users/roles';
		}
		if( $this->permissions->CAN_Edit_Permissions ) {
			$nav['Permissions'] = 'users/permissions';
		}
					  		
		
		$nav['Search']	= 'users/search';
		
		$this->navigation->add($nav);
	}
	
	/** 
	 * Users
	 *	
	 * @access:	public
	 * @uri:	users/
	 * @type:	listing
	 * @post:	displays a list of registered users
	 */
	public function index()
	{
		
		$this->template->write('title', ' Member List');
		
		$page = 0;
		$page = $this->uri->segment(3);
		
		$members = $this->users_model->get_users(25, $page, '', 'userid' ,'ASC');
		
		$roles = $this->permissions->get_roles();		
		$this->data['roles'] = array();

		foreach($roles as $role)
		{
			$this->data['roles'][$role['roleid']] = $role;
		}
		
		$this->data['memberlist'] = "";
		
		if (!empty($members))
		{
			foreach($members as $member)
			{
				$member['role'] = $this->data['roles'][$member['roleid']]['name'];		
				$this->data['memberlist'] .= $this->load->view( $this->mauth->config['template_folder'] . 'member_row', $member, TRUE);					
			}
		}
		else
		{
			$this->data['memberlist'] = 'There are currently active Users in the db.';
		}		
				
		$this->template->write_view('content', $this->mauth->config['template_folder'] . 'members', $this->data);
				
		$this->display();		
	}
	
	
	/** 
	 * Online Users
	 *
	 * @access:	public
	 * @uri:	users/online
	 * @type:	listing
	 * @post:	displays a list of online users
	 */
	public function online()
	{	
		$page = $this->uri->segment(3);
		
		if( empty($page) )	$page = 0;
			
		$online_users = $this->users_model->get_online_users(25, $page);
	
		$this->data['online_users'] = "";
	
        if( ! empty( $online_users ) )
        {
		    foreach($online_users as $user)
		    {	
			    if( empty($user['email']) )		$user['email'] = '';
			
			    $this->data['online_users'] .= $this->load->view($this->mauth->config['template_folder'] .'online_user_row', $user, TRUE);
			
			    $user = "";
		    }
        }

	
		$this->template->write_view('content', $this->mauth->config['template_folder'] . 'online_users', $this->data);
		
		$this->template->write('title', '&mdash; Online Users');
	
		$this->display();
	}
	
	
	/** 
	 * Roles
	 *		 
	 * @access:	public
	 * @uri:	users/roles
	 * @type:	listing
	 * @post:	display a list of roles with options to add / edit.
	 */
	 public function roles()
	{
	 	
	 	$this->load->library('form_validation');
	 	$this->load->helper('form');
	 		 
	 	$this->template->write('title', ' User Roles');
		
	 	$page = array();
	 	$page['role_added'] = FALSE;

			 
	 	$this->form_validation->set_error_delimiters('<span class="error">', '</span>');
		
		$this->form_validation->set_rules('parentid', 'value', 'trim|integer');
		$this->form_validation->set_rules('name', 'Name', 'required|trim');

		if( $this->form_validation->run() != FALSE )
		{
			// add role	
			if( $this->permissions->saveRole(set_value('name'), set_value('parentid')) )
			{
				$page['role_added'] = TRUE;
			}
			
		}
	 	
	 	$roles = $this->permissions->get_roles();
		
		if(!empty($roles)) {
			foreach($roles AS $role) {
				$page['roles'][$role['roleid']] = $role; 
			}
		}
		
	 	$this->template->write_view('content', $this->mauth->config['template_folder'] . 'roles', $page);
	
	 	$this->display();
	 }
	
	/** 
	 * Edit Role
	 *
	 * @access:	public
	 * @uri:	users/editrole/$roleid
	 * @type:	form
	 * @pre: 	Roleid Defined
	 * @post:	form for editing a role displayed.
	 */
	public function editrole()
	{
		$roleid = $this->uri->segment(3);
		if($roleid < 1) redirect('users/roles');
	
		$this->load->library('form_validation');
		$this->load->helper('form');
	
		$this->template->write('title', 'Editing Role');
		
		$this->data['role'] = $this->permissions->get_role($roleid);
		$this->data['permissions'] = $this->permissions->get_permissions();
		
		if( $this->form_validation->run() === FALSE )
		{
		
			$roles = $this->permissions->get_roles();
		
			$this->data['roles'][0] = "NO PARENT ROLE";
		
			if(!empty($roles))
			{
				foreach($roles AS $role) {
					$this->data['roles'][$role['roleid']] = $role['name']; 
				}
			}
			
			$this->template->write_view('content', $this->mauth->config['template_folder'] .'edit_role', $this->data);
		
			$this->display();
		}
		else
		{
			// TODO:	process role edits and update db
		}
	}
	
	/** 
	 * Permissions
	 *
	 * @access:	public
	 * @uri:	users/
	 * @type:	listing
	 * @post:	display a list of permissions with options to add / edit.
	 */
	public function permissions()
	{ 
	 	$this->load->library('form_validation');
	 	$this->load->helper('form');
	 
	 	$this->template->write('title', ' User Permissions');
		$page = array();
	
		$roles = $this->permissions->get_roles();
	
		if(!empty($roles))
		{
			foreach($roles AS $role)
			{
				$page['roles'][$role['roleid']] = $role; 
			}
		}
		
		$this->form_validation->set_error_delimiters('<span class="error">', '</span>');
		
		$this->form_validation->set_rules('uri', 'Related URL', 'trim');
		$this->form_validation->set_rules('description', 'value', 'trim');
		$this->form_validation->set_rules('name', 'Name', 'required|trim|alpha_dash');

		if( $this->form_validation->run() != FALSE )
		{
	 		// save permission
	 		$postroles = $this->input->post('roles');
			
			if( $permid = $this->permissions->save_permission(set_value('name'), set_value('description')) )
			{
				foreach($roles as $role)
				{
					$valid = $postroles["$role[roleid]"];
					
					if($valid < 2)
					{
						$this->permissions->save_uri_permission(set_value('uri'), $role['roleid'], $valid);
						
						$this->permissions->save_role_perm_assoc($role['roleid'], $permid, $valid);
					}
				}
			}

	 		$page['form_saved'] = TRUE;
	 	}
	 	 		
		$page['permissions'] = $this->permissions->get_permissions();

		$this->template->write_view('content', $this->mauth->config['template_folder'] .'permissions', $page);
	 	
	 	$this->display();
	 }
	
	 	
	/** 
	 * Edit User
	 *
	 * @access:	public
	 * @uri:	users/edit
	 * @type:	form
	 * @pre:	$username must be defined
	 * @post:	A users information will be updated
	 */
	public function edit()
	{	
		$username = $this->uri->segment(3);
		
		if($username == '') {
			redirect('users');
		}
		
		$this->template->write('content', 'edit user');
		
		$this->display();
	}

	/**
	 * Delete User
	 *
	 * @access:	public
	 * @uri:	users/deleteuser/$userid
	 * @type:	listing
	 * @pre:	must have priviledges
	 * @param:	int		userid required 	
	 * @post:	Delete a user from the db
	 */
	function deleteuser()
	{
		$userid = $this->uri->segment(3);

		if(!empty($userid) && $this->permissions->CAN_remove_account)
		{
			$this->users_model->remove_user($userid);
		}

		redirect('users');
	}
	
	/** 
	 * --------------------------------------------------------------------
	 * TODO: 
	 *	this has not been fully implemented.  need to place user/ip
	 *	into coventry so they cannot access and/or participate.
	 * --------------------------------------------------------------------
	 *
	 * Ban User
	 *
	 * @access:	public
	 * @uri:	users/ban/$userid
	 * @type:	listing
	 * @pre:	must have significant priviledges.
	 * @param:	int			$userid in uri->segment(3)
	 * @post:	a user will be banned
	 */
	 public function ban()
	 {
	 	
	 	$this->permissions->check('CAN_Ban_User');
	 	
	 	$userid = $this->uri->segment(3);
	 	
	 	$this->template->write('content', 'ban user');
	 	
	 	$this->display();
	 }
	 
	/** 
	 *	Add Account
	 *
 	 * @access:	public
	 * @uri:	users/addaccount
	 * @type:	form
	 * @pre:	must have significant priviledges.
	 * @post:	user account is added
	 */
	public function addaccount()
	{
	  	
		$this->load->library('form_validation');
		
		$this->form_validation->set_error_delimiters('<p class="error">','</p>');
				
		$this->form_validation->set_rules(
				'username', 
				'Username', 
				'required'.
					(isset($this->mauth->config['min_username_length']) ? '|min_length['.$this->mauth->config['min_username_length'] .']' : '') .
					(isset($this->mauth->config['max_username_length']) ? '|max_length['.$this->mauth->config['max_username_length'] .']' : '') .
					'|callback_checkusername'
				);
		$this->form_validation->set_rules('email', 'Email', 'required|valid_email|callback_checkemail');
		$this->form_validation->set_rules('password', 'Password', 'trim|required|matches[passconf]|callback_generate_password');
		$this->form_validation->set_rules('passconf', 'Password Confirmation', 'trim|required');	
		$this->form_validation->set_rules('roleid','Role ID',"required|integer");
		
		if( $this->form_validation->run() == FALSE )
		{
			$data['title'] = 'Add an Accounts';
			$data['tab'] = 'accounts';
			$data['curpage'] = 'viewadmins';

			$roles = $this->permissions->get_roles();
			$data['roles'] = array();
			
			foreach($roles as $role)
			{
				$data['roles'][$role['roleid']] = $role;
			}
						
			$this->template->write_view('content','admin/addaccount', $data, TRUE);
			$this->display();
			
		}
		else
		{
			// form successful, do insert?
			
			$account = array(
							'username'  => set_value('username'),
							'password'  => set_value('password'),
							'email' 	=> set_value('email'),
							'roleid' 	=> set_value('roleid')
							);
			
			$userid = $this->users_model->add_user($account);
			
			redirect('users');
		}
		
	  }
	  
	
	/** 
	 *	Invite User
	 *
	 * @access:	public
	 * @uri:	users/inviteuser
	 * @type:	form
	 * @pre:	must have significant priviledges.
	 * @post:	invite a user to register to the site via email
	 */
	public function inviteuser()
	{
		
		$this->load->library('form_validation');
		
		$this->form_validation->set_error_delimiters('<p class="error">', '</p>');
		
		$this->form_validation->set_rules(
				'username', 
				'Username', 
				'required'.
					(isset($this->mauth->config['min_username_length']) ? '|min_length['.$this->mauth->config['min_username_length'] .']' : '') .
					(isset($this->mauth->config['max_username_length']) ? '|max_length['.$this->mauth->config['max_username_length'] .']' : '') .
					'|callback_checkusername'
				);

		$this->form_validation->set_rules('email', 'Email', 'required|valid_email|callback_checkemail');
		$this->form_validation->set_rules('roleid', 'Role', 'required');
		
		if ($this->form_validation->run() == FALSE)
		{

			$data['title'] = 'Add an Accounts';
			$data['tab'] = 'accounts';
			$data['curpage'] = 'viewadmins';

			$roles = $this->permissions->get_roles();
			$data['roles'] = array();
			
			foreach($roles as $role)
			{
				$data['roles'][$role['roleid']] = $role;
			}
						
			$this->template->write_view('content','admin/inviteaccount', $data, TRUE);

			$this->display();
		}
		else
		{
	
			$this->load->library('email');
		
			$config = $this->config->item('mauth');
			
			$email = set_value('email');
			$username = set_value('username');
			
			$econfig['mailtype'] = 'html';
			$econfig['priority'] = '5';
			
			$this->email->initialize($econfig);
			
			$this->email->from($config['registration']['from_email'] , $this->mauth->config['title']);
			$this->email->to($email);
			
			$this->email->subject( $this->mauth->config['title'] .' >> '. $username .', Your account requires activation');
			
			
			// build account into temp registration table.
			$regcode = $this->mauth->register($username, $email, set_value('roleid'));
			
			
			$message['username'] = $username;
			$message['email'] = $email;
			$message['regkey'] = $regcode;
			
			$email = $this->load->view($this->mauth->config['template_folder'] .'/email/register', $message, TRUE);
			
			$this->email->message($email);
			$this->email->send();
	
			$this->template->write_view('content', $this->mauth->config['template_folder'] .'register/thanks', $message, TRUE);
			
			$this->display();
		}

	}
	
	
	
	/*--------------------------------------------------------------------
	 *	CALLBACK  FUNCTIONS 
	 *-------------------------------------------------------------------*/
	
	/** 
	 * Generate Password
	 *
	 * @access:	private
	 * @type:	function
	 * @post:	a wrapper for generate password
	 */
	function generate_password($password)
	{
		if(empty($password)) return FALSE;
		
		$hash = $this->mauth->make_password($password);
		
		return $hash;
	}
	
	/** 
	 * Callback Username Check
	 *
	 * @access:	private
	 * @params: string		Username required
	 * @return:	Boolean
	 * @post:	Makes sure we have a unique username on registration
	 */
	function checkusername($username)
	{
		
		$isvalid = TRUE;
		
		if($this->users_model->get_user($username)) {
			$isvalid = FALSE;
			$this->form_validation->set_message('checkusername', "Someone has already registered with this %s");
		}
			
		return $isvalid;
	
	}
	
	/** 
	 * Callback Email Check
	 *
	 * @access:	private
	 * @params:	string		email required
	 * @return:	Boolean
	 * @post:	make user the email is unique if config says we require it.
	 */
	function checkemail($email)
	{
		$isvalid = TRUE;
		
		if(!$this->mauth->config['unique_email']) return $isvalid;
		
		if($this->users_model->get_user($email)) {
			$isvalid = FALSE;
			$this->form_validation->set_message('checkemail', "Someone has already registered with this %s");
		}
		return $isvalid;
	}
}

/*	end of file:	users.php
	file location:	./application/controllers/users.php		*/
