<?php if ( !defined('BASEPATH') ) die('No direct script access allowed');

/**-------------------------------------------------------------------------
 *
 * Mauth Configuration File
 * 
 * Edit this file as you please.  The sections are clearly defined
 * and each config should be labeled.  If there is something you
 * don't understand, consult the user guide.
 *
 * @file:  		application/third_party/mauth/config/mauth.php
 * @version:	0.9.2
 * @modified:	October, 2010
 * @author:		Pat Andrew	(http://patandrew.com/)
 * @link:		http://mauth.missionsix.net/
 *
 *-------------------------------------------------------------------------*/
 

/*
|--------------------------------------------------------------------------
| Mauth Site Title
|--------------------------------------------------------------------------
| This is a global variable to hold the name of the site.  You can use if 
| if you want.
|
*/
$config['mauth']['title'] = 'Powered by Mauth';


/*
|--------------------------------------------------------------------------
| Database Table Prefix
|--------------------------------------------------------------------------
| Set this to the table prefix you would like to use for your mauth tables.
|	
| @default:   mauth_
*/
$config['mauth']['table_prefix'] = 'mauth_';

/*
|--------------------------------------------------------------------------
| Users Online Cutoff
|--------------------------------------------------------------------------
| How long until a user is considered inactive on the site?
| 
| @default: 15 minutes
*/
$config['mauth']['online_cutoff'] = 60 * 15;  // 15 minutes.


/*
|--------------------------------------------------------------------------
| Mauth Cookie Settings
|--------------------------------------------------------------------------	
|  Set this to true to allow users to login with cookies data
|
|  note:  May be less secure.   
|  pre-salted, encrypted passwords are stored in cookies
|
| @defaults:
|    use_cookies:    true;
|    cooke_prefix:   mauth_;
|    cooke_path:     /
*/
$config['mauth']['use_cookies']         = TRUE;
$config['mauth']['cookie_prefix']       = 'mauth_';
$config['mauth']['cookie_path']         = '/';
$config['mauth']['cookie_expires']      = 1209600; // 2 * 7 * 24 * 60 * 60 = 2 weeks 

/*
|--------------------------------------------------------------------------
| Mauth Templates Path
|--------------------------------------------------------------------------
| some systems have case sensitive folders, this makes it easier to manage.
|
| @default:	Mauth/	
*/
$config['mauth']['template_folder'] = 'Mauth/';
 
 
/*
|-------------------------------------------------------------------------- 
| Registration Preferences
|--------------------------------------------------------------------------
| variables used in regsitration
| Set this to true to restrict registered accounts to a single email 
| address.
*/
$config['mauth']['reg_from_email']	= 'no-reply@example.com';
$config['mauth']['unique_email'] = TRUE;
   
/*
|--------------------------------------------------------------------------
| Mauth URL preferrences.  
|--------------------------------------------------------------------------
| The following url's are used in helper functions provided by Mauth. 
| They are passed through site_url() prior to being returnded. Include the 
| Mauth Helper file (mauth_helper) in the Mauth package folder.
|
*/
$config['mauth']['login_url']	    = 'auth/login';  // login_url()
$config['mauth']['logout_url']	    = 'auth/logout';  // logout_url()
$config['mauth']['denied_url']      = 'auth/denied';
$config['mauth']['reg_url']	        = 'auth/register'; // registration url
$config['mauth']['comp_reg_url']    = 'auth/confirm_registration'; // email verfication url
$config['mauth']['profile_url']	    = 'profile';
$config['mauth']['settings_url']	= 'profile/settings';


/* 
|--------------------------------------------------------------------------
|	Non-Auth Controllers
|--------------------------------------------------------------------------
| you may add controllers/functions to your site which should not be
| checked against the auth controller.  
|
| @NOTE: Mauth Session data will not be available in your excluded controllers
|
|	Examples:
|   	/controller	
|			would exlude everything running on controller_name
|
|		/controller/function
|			exludes this specific function.  useful for global ajax
|			features because its speeds up processing time.
*/
$config['mauth']['exclude_paths']  = array();

 
/*
|-------------------------------------------------------------------------- 
| Default User ROLE ID
|--------------------------------------------------------------------------
| setting this correctly is VERY important!!!
|
| @default:	1 = user, 2 = admin
|
*/
$config['mauth']['user_roleid'] = 1;


/*
|--------------------------------------------------------------------------
| Username Length Preferences
|--------------------------------------------------------------------------
| set to username designed username lengths
|
| Comment lines to not check username length (not recommended)
*/
$config['mauth']['min_username_length'] = 4;
$config['mauth']['max_username_length'] = 16;
 
 



##------------------------------------------------------------------------
##
##	Facebook Connect & Facebook Applications Config
##
##------------------------------------------------------------------------

/*
|-------------------------------------------------------------------------- 
|  Use Facebook Login?
|--------------------------------------------------------------------------
| set to true if you would like to add a Facebook login feature
| This is useful for Facebook Connect or Facebook Applications
|	
|	 @default: false;
*/
$config['mauth']['facebook']['enabled'] = false;

  
/*
|-------------------------------------------------------------------------- 
| Facebook Application Key & Secret Key   (optional)
|--------------------------------------------------------------------------	
| Required only if you intent on using a facebook login.	
|
| Change these config vars to match your facebook application key
*/
$config['mauth']['facebook']['app_key'] = ''; 
$config['mauth']['facebook']['secret'] = '';
$config['mauth']['facebook']['generate_secret'] = false;	


/* 
|--------------------------------------------------------------------------
| Facebook Registration URL
|--------------------------------------------------------------------------
| The url where a user is redirected in order to create an account.
|
| @note:  this url could process a fb username and automatically create an account
|           for the user, or you can ask a user to create an account.
*/
$config['mauth']['fb_reg_url'] = 'auth/fb_reg';  
 	
 
/* end of file: mauth.php	*/
/* location: ./application/config/mauth.php		*/
