<?php  if (!defined('BASEPATH')) exit('No direct script access allowed');

/**-------------------------------------------------------------------------
 *
 *	Navigation Model
 *	
 *	Provides a simple breadcrumbs and nav links, used for secondary 
 *	navigational items. Primary navigation should be dealt with in the views
 *
 * @file:  		application/third_party/mauth/models/navigation.php
 * @version:	0.9.2
 * @modified:	October, 2010
 * @author:		Pat Andrew	(http://patandrew.com/)
 * @link:		http://mauth.missionsix.net/
 *
 *-------------------------------------------------------------------------*/


class Navigation extends CI_Model
{

	private $crumbs = array();	
	private $navs = array();
	
	
	public function __construct() {
		
		parent::__construct();
		
		// fake breadcrumbs
			
		$this->crumb('', 'Home');
		
		$segments = $this->uri->segment_array();
			
		foreach($segments as $key=>$segment)
		{
  	    	 $this->crumb(implode( '/', array_slice($segments,0,$key)), $segment);
		} 
		
	}
	
	/** 
	 *	Add Breadcrumb
	 *	allows you to add a breadcrumb to the trail
	 *
	 *  @access     public
     *  @param      string/array    crumb URL or array of crumbs
     *  @param      string          crumb text 
	 */
	public function crumb($url, $crumb = '') {
		
        if( is_array($url) )
        {
            foreach($url as $u => $c) {
                $this->crumb( $u, $c );
            }
        } else
        {    
    		$this->crumbs["{$url}"] = $crumb;
        }
	}
	
	/**
	 *	Eat Breadcrumbs
	 *	removes all of the current breadcrumbs.
	 *
     *  @access     public
	 */
	 public function eat() {
	 	$this->crumbs = array();
	 }
	 
	/**
	 *	Chew Breadcrumbs
	 *	Removes a specific crumb from the trail.
	 *
     *  @access     public
	 */
	public function chew($url) {
		unset($this->crumbs["{$url}"]);
	}
	
	
	/** 
	 *	Add navigation 
	 *	allows you to add a navigational item, this is useful for Secondary Navigation!!
	 *
	 *  note:  is recursive if you pass an array as the first paramenter (second is ignored)
	 *
     *  @access     public
	 *	@param      string      anchor text
	 *	@param      string      url
	 */
	public function add($text = '', $nav = '') {
		
		if( is_array($text) ):
			foreach($text as $k => $v):
				$this->add($k, $v);
			endforeach;
		else:
			$this->navs["$nav"] = $text;
	
		endif;
	}
	

	/** 
	 *	get Navigation 
	 *	allows you to get the navigation array
     * 
     *  @access     public
     *  @post       array       navigation array
	 */
	public function get() {
		return $this->navs;
	}


	/** 
	 *	Trail
     *  returns a string of breadcrumbs 
     *
     *  @access     public
     *  @post       string      breadcrumb anchors.
	 */
	public function trail() {
		
		$breadcrumbs = $this->crumbs;
		
		$i = 0;
		$output = '';
		
		// print
		foreach($breadcrumbs as $url => $txt) 
        {
			$i++;
            $extras = ( count($breadcrumbs) === $i ? array('class'=>'here') : array() );
			$output .= anchor( $url, $txt, $extras ); 
			
			if(count($breadcrumbs) != $i)
                $output .=" &raquo; ";
		}
		
		return $output;
	}
}
