<?php if ( !defined('BASEPATH') ) die('No direct script access allowed');

/**-------------------------------------------------------------------------
 *
 *	Permissions Model
 *	
 *	Provides methods for access restriction and querying the model.
 *
 * @file:  		application/third_party/mauth/models/permissions.php
 * @version:	0.9.2
 * @modified:	October, 2010
 * @author:		Pat Andrew	(http://patandrew.com/)
 * @link:		http://mauth.missionsix.net/
 *
 *-------------------------------------------------------------------------*/


class Permissions extends CI_Model
{
 	
 	// permissions store,  only accessible through php 5's __get()
 	private $perms = array();
 	
 	private $conf;
 	
 	/**
 	 *
 	 *	Construct 
 	 *	loads the current permissions for a user's role.
 	 *
 	 */
 	public function __construct()
	{
 		parent::__construct();
 		
 		$this->conf = $this->config->item('mauth');
 		
 		$this->load_permissions( $this->session->userdata('roleid') );
 		
 	}
 	
 	/** 
 	 *	Public Check
 	 *	users permission properties to check if a user can do something.
 	 *
 	 *	@pre:	permission value is passed, eg  'permission_name'	
 	 *	@post:	Returns a boolean indicating if the user has the permission
 	 */
 	public function check($permission = '') {
 	
 		if(empty($permission)) return false;
 		
 		$can = null;
 		
 		// recursive
 		if(is_array($permission)) {
 			foreach($permission as $perm) {
 				$can = $this->check($perm);
 			}
 		} else {
 			$can = $this->$permission; // using PHP5's __get()
 		}
 		
 		return $can;
 	}
 	
 	
 	/** 
 	 *	Public Verify Route
 	 *	uses roles & routes to verify user access granted in role_uri table.
 	 *	routes in table are absolute, meaning they rely on rsegments.
 	 *
 	 *	@post:	Route is either continued, or user is redirected to login/denied accordingly
 	 *
 	 */
 	public function verify_route()
    {
	 	$canroute =  $this->get_role_uri( $this->session->userdata('roleid') );
 	
 		if($canroute < 2) {
 			if($canroute == 0) {
                                log_message('debug', 'User denied access. IP: '. $this->mauth->ipaddress);
 				redirect($this->conf['denied_url']);
 			}
 		}
 	}
 	
 	/**
 	 *	Get Role
 	 *	a role uri is grabbed from the db, if its there, we assume access is restricted for everyone but that role
 	 *	
 	 *	@post:	a value index is returned (0..2) 
 	 *			2 = don't care
 	 *			1 = valid
 	 *			0 = restricted
 	 */
 	public function get_role_uri($roleid = 0)
    {
 		
 		$uri_segs = $this->uri->rsegments; 
 		
 		$table_prefix = $this->conf['table_prefix'];
 		
 		$segments = $this->uri->segment_array();
 		$uris = '';
 		if(!empty($segments)) {
			foreach($segments as $key=>$segment)
			{
				// $uris .= " XOR uri ='". substr($this->uri->uri_string, 1) ."'";
  			 	 $uris .= " XOR uri ='". implode( '/', array_slice($segments,0,$key)) ."'";
			} 
		}
		
 	 	$valid = 2;
			 		
	 	// get uri_role permission for current URI.
	 	
	 	$sql = "SELECT * FROM {$table_prefix}role_uri 
	 			WHERE (uri = '". $uri_segs[1] ."'{$uris})";
	 		
	 	$res = $this->db->query($sql);
	 	
	 	$roles = $res->result_array();
	 		
	 	if($roles) { 			
	 		$valid = 0;
	 		$checkparentrole = true;
	 		
	 		foreach($roles as $role) {
	 			if($role['roleid'] == $roleid) {
	 				$valid = $role['valid'];
	 				$checkparentrole = false;
	 			}
	 		}	
	 		 		
	 		if($checkparentrole) {
                log_message('debug', 'MAUTH:	checking parent roleid');

		 		// get role parents
		 		$sql = "SELECT parentid FROM {$table_prefix}roles WHERE roleid={$roleid}";
		 		
		 		$res = $this->db->query($sql);
 	
		 		$parent = $res->row_array();
		 		
		 		if($parent) {
		 		
		 			if($parent['parentid'] > 0) {
		 				$valid = $this->getRole($parent['parentid']);			
		 			}
		 		}
	 		
	 		}	
	 	}

	 	return $valid;
	 }


 	/** 
	 *	Get Role
	 *
	 *	@pre:	(int) $roleid
	 *	@post:	returns a roles details	
	 */
    public function get_role($roleid = 0)
    {
		if($roleid < 1) return false;
		
                $table_prefix = $this->conf['table_prefix'];
		
                $sql = "SELECT * FROM {$table_prefix}roles WHERE roleid = $roleid LIMIT 1";
		$res = $this->db->query($sql);

		if($res) {
			return $res->row_array(); 
		}
		return false;
	 }
	 
	 
 	/** 
 	 *	Load Permissions
 	 *	loads $this->data with permissions for the current logged in user.
 	 *
 	 */
 	private function load_permissions($roleid = 0)
    {
        log_message('debug', "MAUTH:	Loading Permissions for role #$roleid");	 	

 		// if($roleid < 1) return false;
 		
 		$perms = $this->get_permission($roleid);
 		
 		if($perms) {
 			foreach($perms as $k => $v) {	
 				$this->$k = $v;
 			}
 		}
 	}
 	
 	
 	/** 
 	 *	Get Permissions
 	 *	Returns an array of permissions for a given role ID.   
 	 *  If a role has a parent, a recursive call to getPermissions() is performed
 	 *	which is the base permissions for the current role id.  
 	 *
 	 * 	@params: roleid
 	 *	@return:  Array indexed by permissions names of BOOLEANs.
 	 *				ex:		$perms['CAN_EDIT_STYLES'] = TRUE;
 	 */
 	
 	public function get_permission($roleid = 0)
    {
 		
 		if($roleid == 0) return false;
 
 log_message('debug', "MAUTH:	Getting Permission for role $roleid");	 	
 		
 		// permissions
 		$perms = array();
 		$parentperms = array();
 		
 		
 		$table_prefix = $this->conf['table_prefix'];
 		
 		// get role parents
 		$sql = "SELECT parentid FROM {$table_prefix}roles WHERE roleid={$roleid}";
 		$res = $this->db->query($sql);
 		
 		$parent = $res->row_array();
 		
 		if(!$parent) return false;
 		
 		if(!empty($parent)) {
	 		if($parent['parentid'] != 0) {
	 			$parentperms = $this->get_permission($parent['parentid']);
	 			if(is_array($parentperms)) {
		 			// overload & maintain previous permissions.
		 			foreach($parentperms as $k => $v) {
		 				$perms[$k] = $v;
		 			}
	 			}
				else
				{
	 				die('Failed to properly load parent permissions.');
	 			}
 			}
 		}
 		
 		// get permissions.

 		$sql = "SELECT {$table_prefix}permissions.*, {$table_prefix}role_permissions.value
 				FROM {$table_prefix}role_permissions
 				LEFT JOIN {$table_prefix}permissions ON ({$table_prefix}permissions.permid = {$table_prefix}role_permissions.permid) 
 				WHERE {$table_prefix}role_permissions.roleid = $roleid";
 		
 		$res = $this->db->query($sql);
 		
 		if(!$res) return false;
 		
 		$results = $res->result_array();
 		
 		if(!empty($results)) {
 		
 			foreach($results AS $permission) {
 				$perms['CAN_'. $permission['title']] = $permission['value']; 
 			}
 		}
 		
 		return $perms;
 	}
 	
 	
 	
 	/**
 	 *	Permissions
 	 *	returns an array of all permissions with sub-arrays of rolenames & roleids
 	 *
 	 */
 	public function get_permissions()
	{
 		
 		$table_prefix = $this->conf['table_prefix'];
 		
 		$sql = "SELECT {$table_prefix}permissions.*, {$table_prefix}role_permissions.value, 
 				GROUP_CONCAT({$table_prefix}roles.name SEPARATOR ',') AS rolenames,
 				GROUP_CONCAT({$table_prefix}roles.roleid SEPARATOR ',') AS roleids,
 				GROUP_CONCAT({$table_prefix}role_permissions.value SEPARATOR ',') AS rolevalues
 				FROM {$table_prefix}role_permissions
 				LEFT JOIN {$table_prefix}permissions ON ({$table_prefix}permissions.permid = {$table_prefix}role_permissions.permid)
 				INNER JOIN {$table_prefix}roles ON ({$table_prefix}roles.roleid = {$table_prefix}role_permissions.roleid)
 				GROUP BY {$table_prefix}permissions.permid 
 				";
 		
 		$res = $this->db->query($sql);
 		
 		if(!$res) return false;
 		
 		$permissions = $res->result_array();
 		
 		foreach($permissions as $key => $permission)
		{
	 		$permissions[$key]['rolenames'] = explode(',', $permission['rolenames']);
	 		$permissions[$key]['roleids'] = explode(',', $permission['roleids']);
            $permissions[$key]['rolevalues'] = explode(',', $permission['rolevalues']);
	 	}	 	
 		return $permissions;
 	}
 	
 	/** 
 	 *	Get Roles
 	 *	returns an array of roles / titles 
 	 *
 	 */
 	 public function get_roles() {
 	 
 	 	$table_prefix = $this->conf['table_prefix'];
 		
 		$sql = "SELECT {$table_prefix}roles.* FROM {$table_prefix}roles";
 		
 		$res = $this->db->query($sql);
 		
 		if(!$res) return false;
 		
 		return $res->result_array();
 	 }
 	  	
 	
 	/**
 	 *	Save Permissions
 	 *	allows you to save permissions to the database
 	 *
 	 *	@pre:	permission title is defined, description can be empty;
 	 *
 	 *	@post:	permission is saved in db. insert id is returned
 	 */
 	public function save_permission($title = '', $description = '')
	{
 		
 		if(empty($title)) return false;
 	
 		$table_prefix = $this->conf['table_prefix'];
 		$sql = "REPLACE INTO {$table_prefix}permissions (title, description) VALUES ('$title', '$description')";
 		
 		$res = $this->db->query($sql);
 		
 		if(!$res) return false;
 		
 		return $this->db->insert_id();
 		
 	}
 	
 	/** 
 	 *	Save Permission Role Assoc
 	 *	pass a roleid, permid and value 
 	 *
 	 */
 	public function save_role_perm_assoc($roleid = 0, $permid = 0, $value = 0)
	{
 	
 		$table_prefix = $this->conf['table_prefix'];
 		$sql = "REPLACE INTO {$table_prefix}role_permissions (roleid, permid, value)
                VALUES (". $this->db->escape($roleid) .",". $this->db->escape($permid) .",". $this->db->escape($value). ")";
 	
 		$res = $this->db->query($sql);
 		
 		if(!$res) return false;
 		
 		return true;
 	}
 	
 	
 	/** 
 	 *	Save URI Permission
 	 *	
 	 *	@pre:	permid, uri, value are defined
 	 *	@post:	saved in db.
 	 *
 	 */
 	public function save_uri_permission($uri = '', $roleid = 0, $value = 0)
	{
 	
 		if(empty($uri)) return false;
 		
 		$table_prefix = $this->conf['table_prefix'];
 		$sql = "REPLACE INTO {$table_prefix}role_uri (roleid, uri, valid) VALUES ('$roleid','$uri','$value')";
 	
 		$res = $this->db->query($sql);
 		
 		if(!$res) return false;
 		
 		return true;
 	}
 	
 	
 	/** 
 	 *	Add Roles
 	 *	add a new role to the db
 	 */
 	public function save_role($name = '', $parentid = 0)
	{
 		
 		if(empty($name)) return false;
 		
 		$table_prefix = $this->conf['table_prefix'];
 		$sql = "INSERT INTO {$table_prefix}roles (name, parentid) VALUES ('". strtolower($name) ."','$parentid')";
 	
 		$res = $this->db->query($sql);
 		
 		if(!$res) return false;
 		
 		return true;
 		
 	}
 	
 	
 	
 	/**
 	 *	Getter
 	 *	provides access to $data
 	 *  must provide CAN_  in front of permission name.
 	 *
 	 *	use:   		$permissions->VAR
 	 *	returns:	$this->data[VAR] OR false.
 	 */
	public function __get($name) {
	     
		$name = $this->_stripCan($name);
	      
	    if (array_key_exists($name, $this->perms)) {
            return $this->perms[$name];
        } elseif(isset($this->$name)) {
        	return $this->$name;
        } else {
        	return NULL;
        }
	}

 	
 	/**
 	 *	Permissions Setter
 	 *	provides write access to $data
 	 *  must provide CAN_  in front of permission name.
 	 *
 	 *	use:   		$permissions->VAR
 	 *	returns:	$this->data[VAR] OR false.
 	 */ 	
 	public function __set($name, $value = TRUE)
	{
 		$name = $this->_stripCan($name);

		if (array_key_exists($name, $this->perms))
		{
                    $this->perms[$name] = $value;
                    return $value;
            } else {
        	    $this->$name = $value;
                    return $value;
            }
 	}
 	

    public function __isset($name) {
    	$name = $this->_stripCan($name);
        return isset($this->data[$name]);
    }

    
    public function __unset($name) {
        $name = $this->_stripCan($name);
        unset($this->data[$name]);
    }
    
    
    
    /** 
     *	Private Function Strip Can
     *	removes CAN_ from permission name
     *  @return: String;
     */
    private function _stripCan($name) {
    	$prefix = substr($name, 0,4);
    	
    	if($prefix == 'CAN_') { 
    		return strtolower(substr($name, 4));
    	}
    	return strtolower($name);
    }
 }

/* End of file permissions.php */ 
/* Location: ./application/models/permissions.php */ 
