<?php  if (!defined('BASEPATH')) exit('No direct script access allowed');

/**-------------------------------------------------------------------------
 *
 *	Mauth Musers model
 *	
 *	This model provides an interface to the Mauth user tables.
 *
 * @file:  		application/third_party/mauth/models/users_model.php
 * @version:	0.9.2
 * @modified:	October, 2010
 * @author:		Pat Andrew	(http://patandrew.com/)
 * @link:		http://mauth.missionsix.net/
 *
 *-------------------------------------------------------------------------*/


class Users_Model extends CI_Model
{	
	public $profiles = array();
	public $users = array();
	
	private $table_prefix;
	private $user_roleid = 1;

	/**
 	 * Constructor
	 * sets some default strings so the entire config item doesn't have
	 * to live here.
	 *
 	 * @access:		public
	 */
	function __construct() {
		
		parent::__construct();
        
        $conf = &$this->config->item('mauth');        
		$this->table_prefix = $conf['table_prefix'];
        $this->user_roleid = $conf['user_roleid'];
	}
	
	/**
	 * Get Profiles	
	 *
	 * @access:		public
	 * @return:		array		profile array
	 */
	function get_profiles() {
		$sql = "SELECT * FROM ". $this->table_prefix ."profiles";
		
		$res = $this->db->query($sql);
		$profiles = $res->result_array();
		
		$profilelist = array();
		foreach($profiles as $k => $v) {
			$this->profiles[$v['userid']] = $v;			
		}
        return $this->profiles;
	}
	
	
	/** 
	 * Add A User
	 * Saves a user and thier information to the database.
	 *
     * @access		public
	 * @param:		Array      user account details 
	 * @return:		integer     user id of 
     *    
     * TODO:  validate inputs better.
	 */
	function add_user($user = array()) {
		if(empty($user)) return false;
		
		if(empty($user['roleid'])) {
			$user['roleid'] = $this->user_roleid;
		}
		
		$this->db->insert($this->table_prefix .'users', $user);
	
		return $this->db->insert_id();
	}
	 
	/** 
	 * Remove User Account
	 *
     * @access:     public
	 * @param:      integer     userid supplied
	 * @post:       user is removed from db.
	 */ 
	function remove_user($userid = 0)
	{
		if($userid < 1) return false;
		
			$tmpsql = "SELECT roleid FROM ". $this->table_prefix ."users WHERE userid = $userid LIMIT 1";
			$tmp = $this->db->query($tmpsql);
			$tmparray = $tmp->row();

			$cur_userid = $this->session->userdata('roleid');

			if($tmparray->roleid != 2 && $cur_userid != $userid) {
				$sql = "DELETE FROM ". $this->table_prefix ."users WHERE userid = $userid";
				$this->db->query($sql);
				return true;
			}
		return false;
	}
	
	 
	/**
	 * Get Users
	 *
	 * @access:		public		
	 * @param:  	integer		how many users to return
	 * @param:		integer		which page of results
	 * @param:		string		any mysql conditions for filtering results
	 * @param:		string		column to sort on		
	 * @param:		string		'ASC' | 'DESC' sort order
	 * @return		array		a list of users
	 */
	function get_users($limit = 10, $page = 0, $condition = '', $orderby = 'userid', $direction = 'DESC') {
	
		$limiter = '';
		
		if($page > 0) {
			$start = $page * $limit - 1;
		}
		
		if($limit > 0) {
			$limiter = 'LIMIT $limit, $start';
		}
	
		$res = $this->db->query("SELECT ". $this->table_prefix ."users.*
								 FROM   ". $this->table_prefix ."users 
								 $condition
								 ORDER BY $orderby $direction");
		
		if($res->num_rows() > 0) {
			// we have members, lets return them
			
			$users = $res->result_array();
			$profiles = array();
			foreach($users as $user) {
				//$avatar = $this->getAvatar($user['userid']);
				$profile = $this->get_profile($user['userid']);
				$profile['profile'] = substr($profile['profile'],0, 250) . '[...]';
				
	            /*
                if(is_array($avatar)) {
					$profile = array_merge($avatar, $profile);
				}
                */				
				$profiles[] = array_merge($user,$profile);
			}
			
			return $profiles;
		} else {
			return false;
		}
	}
	
	
	/** 
	 *	Get Online Users
	 *	returns an array of current online users.  If a userid is associated, return the username.
	 *
	 * @access: 	public
	 * @param:  	integer		how many users to return
	 * @param:		integer		which page of results
	 * @param:		string		any mysql conditions for filtering results
	 * @param:		string		column to sort on		
	 * @param:		string		'ASC' | 'DESC' sort order
	 * @post: 		array		online users is returned
	 */
	function get_online_users($limit = 10, $page = 0, $where = "", $orderby = 'userid', $direction = 'DESC')
    {
	
		$limiter = '';
		
		if($page > 0) {
			$start = $page * $limit - 1;
		}
		
		if($limit > 0) {
			$limiter = 'LIMIT $limit, $start';
		}	
		
		// $time_cutoff = date("Y-m-d H:i:s", $this->mauth->activity - $this->mauthconf['online_cutoff']);
		$conf = $this->config->item('mauth');
		$time_cutoff = $this->mauth->activity - $conf['online_cutoff'];
		
		$condition = "WHERE last_activity >= $time_cutoff";
	
		$sql = "SELECT ". $this->table_prefix ."sessions.*
								 FROM   ". $this->table_prefix ."sessions 
								 $condition 
								 ORDER BY $orderby $direction";
		
		$res = $this->db->query($sql);

		if($res->num_rows() > 0)
        {
			$users = $res->result_array();
			$onlineusers = array();
			
            foreach($users as $k => $user)
            {
				$userinfo = unserialize($user['user_data']);

				if($userinfo['userid'] < 1)
				{
					$user['username'] = 'Guest';
                    unset($user['user_data']);
					$onlineusers[$k] = $user;
				} else
				{					
                    // remove unnecessary cruft from array
                    unset($user['user_data'], $userinfo['logged_in'],
                          $userinfo['password'], $userinfo['avatar'], $userinfo['profile']);

					$onlineusers[$k] =  array_merge($user, $userinfo);	
				}
			}
			
			return $onlineusers;
		}
		return FALSE;
	}


	/**
	 *	Get User
	 *	user the optional parameter of $facebook to trigger a facebook user lookup
	 *	
	 * @access:		public
	 * @param:		string|integer		user name, userid, fbuserid, or email address
	 * @param:		boolean				if set to True, you can look up on facebook userid
	 * @return:		array 				user data including: username, userid, email
	 */
	function get_user($user = '', $facebook = false) {
		
		$this->load->helper('email');
		
		if(!empty($user)) {
			$where = '';
			if($facebook && is_numeric($user)) {
				$where = "fbuserid= '". $user."'";
			} elseif(is_numeric($user)) {
				$where = "userid='". $user ."'";
			} elseif(valid_email($user)) {
				$where = "email ='". $user ."'";
			} else {
				$where = "username='". $user ."'";
			}	
			$res = $this->db->query("SELECT username, password, userid, email, fbuserid, profile, roleid  
											FROM   ". $this->table_prefix ."users WHERE $where");
			if($res->num_rows() > 0) {
				$entry = array();
				$entry = $res->row_array();
				return $entry;
			} else {
				 return false;
			}
		
		} else {
			return false;
		}

	}


	/**
	 * Get Profile
	 * allows you to get a user's profile
	 *
	 * @access:		public
	 * @param:		integer		user id
	 * @return:		array		profile information
	 */
	 
	 function get_profile($userid = 0) {
	 
	 	if($userid > 0) {
	 		if(!empty($this->profiles[$userid])) {
				return $this->profiles[$userid];
			} else {
	 			$profile =	$this->db->query("SELECT * FROM ". $this->table_prefix ."profiles WHERE userid=$userid LIMIT 1");
	 			if($profile->num_rows() > 0) {
					$prof = $profile->result_array();
					$this->profiles[$userid] = $prof[0];
					return $prof[0];
				} else {
						// we don't have a profile, lets make one!
						$profile = $this->db->query("INSERT INTO ". $this->table_prefix ."profiles (userid) VALUES ($userid)");
						//return false;
				}
			}
	 	} else {
	 		return false;
	 	}
	 }
	 
	/** 
	 * @DEPRECIATED
	 *	function saveAvatar
	 *	Allows you to save an avatar and it will store correctly
	 *
	 *	@params: 	INT 		$userid
	 *				[@PATH] 	$avatar  (final image path)
	 *
	 *	@return:	BOOL	
	 
	 function saveAvatar($userid = 0, $avatar = '') {
	 	
	 	if($userid > 0 && is_array($avatar)) {
	 	
	 		$saveimage = $this->db->query("INSERT INTO ". $this->mauthconf['cookie_prefix'] ."images (loc, data) VALUES ('". $avatar['file_name'] ."','". serialize($avatar) ."')");
	 		if(!$saveimage) {
	 			return false;
	 		}
	 		
	 		$imageid = $this->db->insert_id();
	 		
	 		$user = $this->getProfile($userid);
	 		
	 		$saveava = $this->db->query("INSERT INTO ". $this->mauthconf['cookie_prefix'] ."profile_images (profileid, imageid) VALUES ($user[profileid], $imageid)");
	 		
	 		if($saveava) {
	 		
	 			// good, we have an avatar, lets resize it!
	 			
	 			$config['image_library'] = 'gd2';
				$config['source_image'] = $avatar['full_path'];
				$config['create_thumb'] = TRUE;
				$config['maintain_ratio'] = TRUE;
				$config['width'] = $this->mauthconf['max_avatar_width'];
				$config['height'] = $this->mauthconf['max_avatar_height'];

				$this->load->library('image_lib', $config);

				if(!$this->image_lib->resize()) {
					return false;
				}
	 				
	 			return true;
	 		}
	 	}
	 	return false;
	 }
	 
	 /**
	  *	Get Avatar
	  *	Allows you to get a user's avatar 
	  *
	  *	@params: 	INT		$userid;
	  

	function getAvatar($userid = 0) {
		if($userid > 0) {
			
			$user = $this->getProfile($userid);
			
			if(!$user) {
				return false;
				}
			$res = $this->db->query("SELECT ". $this->table_prefix ."profile_images.*, ". $this->table_prefix ."images.* 
									 FROM ". $this->table_prefix ."profile_images 
									 LEFT JOIN ". $this->table_prefix ."images 
									 ON (". $this->table_prefix ."profile_images.imageid = ". $this->table_prefix ."images.imageid)
									 WHERE profileid=". $user['profileid'] ." ORDER BY profileimageid DESC LIMIT 1");
			
			if($res->num_rows() > 0) {
				$ava = $res->result_array();
				
				$img = unserialize($ava[0]['data']);
				$avatar['avatar'] = '/userfiles/'. $img['raw_name'] .'_thumb'. $img['file_ext'];
				
				return $avatar;
			} else {
				// show default image;
			
				$avatar['avatar'] = '/images/avatar.gif';
				return $avatar;
			}
			
		} else {
			return 'no userid supplied';
		}
	}
    */
}

	
